/*
 * @Author: your name
 * @Date: 2021-12-03 14:41:52
 * @LastEditTime: 2022-03-16 15:33:48
 * @LastEditors: Please set LastEditors
 * @Description: 打开koroFileHeader查看配置 进行设置:
 * https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
 * @FilePath: /EHR/home/igs/roadmodel/src/CtApRoadModel/source/road/lane.cc
 */
#include "lane.h"

#include <iostream>
namespace rm {

void Lane::set_lane_id(const uint8_t &lane_id) { lane_id_ = lane_id; }

void Lane::set_lane_position(const LanePosition &lane_position) {
  lane_position_ = lane_position;
}

void Lane::set_start_offset(const float &start_offset) {
  start_offset_ = start_offset;
}

void Lane::set_end_offset(const float &end_offset) { end_offset_ = end_offset; }

void Lane::add_left_lane_marker_id(const uint32_t &lane_marker_segment_id) {
  left_lane_marker_id_.emplace_back(lane_marker_segment_id);
}

void Lane::add_right_lane_marker_id(const uint32_t &lane_marker_segment_id) {
  right_lane_marker_id_.emplace_back(lane_marker_segment_id);
}

LaneSegment *Lane::add_front_lane_segment() {
  front_lane_segment_.emplace_back();
  return &front_lane_segment_.back();
}

LaneSegment *Lane::add_back_lane_segment() {
  back_lane_segment_.emplace_back();
  return &back_lane_segment_.back();
}

LaneSegmentAttribute *Lane::add_lane_segment_attribute() {
  lane_segment_attribute_.emplace_back();
  return &lane_segment_attribute_.back();
}

void Lane::set_is_start_out_of_range(const bool &is_start_out_of_range) {
  is_start_out_of_range_ = is_start_out_of_range;
}

void Lane::set_is_end_out_of_range(const bool &is_end_out_of_range) {
  is_end_out_of_range_ = is_end_out_of_range;
}

void Lane::set_is_left_lane_marker_changed(
    const bool is_left_lane_marker_changed) {
  is_left_lane_marker_changed_ = is_left_lane_marker_changed;
}

void Lane::set_is_right_lane_marker_changed(
    const bool is_right_lane_marker_changed) {
  is_right_lane_marker_changed_ = is_right_lane_marker_changed;
}

uint8_t Lane::lane_id() const { return lane_id_; }

LanePosition Lane::lane_position() const { return lane_position_; }

float Lane::start_offset() const { return start_offset_; }

float Lane::end_offset() const { return end_offset_; }

const std::list<uint32_t> &Lane::left_lane_marker_id() const {
  return left_lane_marker_id_;
}

const std::list<uint32_t> &Lane::right_lane_marker_id() const {
  return right_lane_marker_id_;
}

std::list<uint32_t> *Lane::mutable_left_lane_marker_id() {
  return &left_lane_marker_id_;
}

std::list<uint32_t> *Lane::mutable_right_lane_marker_id() {
  return &right_lane_marker_id_;
}

std::ostream &operator<<(std::ostream &os, const Lane &lane) {
  os << "\n"
     << "lane_id_ = " << +lane.lane_id_ << "\n"
     << "lane_position_ = " << (+uint8_t(lane.lane_position_)) << "\n";
  os << "left_lane_marker_segment_id_ = ";
  for_each(lane.left_lane_marker_id_.begin(), lane.left_lane_marker_id_.end(),
           [&os](uint32_t id) { os << id << " "; });
  os << "\n";
  os << "right_lane_marker_segment_id_ = ";
  for_each(lane.right_lane_marker_id_.begin(), lane.right_lane_marker_id_.end(),
           [&os](uint32_t id) { os << id << " "; });
  return os;
}

void Lane::set_lane_type_and_speed_limit() {
  if (lane_segment_attribute_.empty()) {
    lane_type_ = rm::LaneType::UNKNOWN;
    speed_limit_.first = 0.0f;
    speed_limit_.second = 0.0f;
  } else {
    auto it = std::find_if(
        lane_segment_attribute_.begin(), lane_segment_attribute_.end(),
        [](const LaneSegmentAttribute &seg) {
          return (0.0 < seg.end_offset() && 0.0 >= seg.start_offset());
        });
    if (it != lane_segment_attribute_.end()) {
      lane_type_ = it->lane_type();
      speed_limit_.first = it->max_speed_limit();
      speed_limit_.second = it->min_speed_limit();
    } else {
      if (0.0 < lane_segment_attribute_.front().start_offset()) {
        lane_type_ = lane_segment_attribute_.front().lane_type();
        speed_limit_.first = lane_segment_attribute_.front().max_speed_limit();
        speed_limit_.second = lane_segment_attribute_.front().min_speed_limit();
      } else {
        lane_type_ = lane_segment_attribute_.back().lane_type();
        speed_limit_.first = lane_segment_attribute_.back().max_speed_limit();
        speed_limit_.second = lane_segment_attribute_.back().min_speed_limit();
      }
    }
  }
}

float Lane::lane_width() const {
  return front_lane_segment_.empty() ? 0.
                                     : front_lane_segment_.front().lane_width();
}

float Lane::lane_merge_distance() const {
  // if (is_end_out_of_range_) {
  //   return 1000.0F;
  // } else {
  //   auto it = std::find_if(
  //       lane_segment_attribute_.begin(), lane_segment_attribute_.end(),
  //       [](const LaneSegmentAttribute &seg) {
  //         rm::TransitType trans_type = seg.transit_type();
  //         return (trans_type == TransitType::LEFTENTERBEGIN) ||
  //                (trans_type == TransitType::RIGHTENTERBEGIN) ||
  //                (trans_type == TransitType::LEFTENTEREND) ||
  //                (trans_type == TransitType::RIGHTENTEREND) ||
  //                (trans_type == TransitType::MERGE);
  //       });
  //   if (it != lane_segment_attribute_.end()) {
  //     return it->start_offset();
  //   } else {
  //     return 1000.0F;
  //   }
  // }
  if (is_end_out_of_range_) {
    return 1000.0F;
  } else {
    return end_offset_;
  }
}

}  // namespace rm